@@ -12,36 +12,47 @@
 
 package Badger::Config;
 
-use Badger::Debug ':dump';
+use Badger::Debug ':dump debugf';
 use Badger::Class
     version   => 0.01,
     debug     => 0,
     import    => 'class',
     base      => 'Badger::Prototype',
-    utils     => 'blessed numlike',
+    utils     => 'blessed numlike extend',
     constants => 'HASH ARRAY CODE DELIMITER',
     auto_can  => 'can_configure',
+    alias     => {
+        init  => \&init_config,
+    },
     messages  => {
-        get => 'Cannot fetch configuration item <1>.<2> (<1> is <3>)',
+        get       => 'Cannot fetch configuration item <1>.<2> (<1> is <3>)',
+        no_config => 'No configuration data found for %s',
     };
 
 
-sub init {
+
+sub init_config {
     my ($self, $config) = @_;
-    my $data  = $self->{ data } = $config->{ data } || $config;
+    my $data  = $self->{ data } = $config->{ data } || { %$config };
     my $class = $self->class;
-    
-    # merge all $ITEMS in package variables with those listed in 
+
+    # merge all $ITEMS in package variables with those listed in
     # $config->{ items } and all other $config keys.
-    my $items = $self->class->list_vars( 
+    my $items = $class->list_vars(
         ITEMS => delete($config->{ items }), keys %$data
     );
-    
+
+    if (DEBUG) {
+        $self->debug("[$self] $class ITEMS: ", $self->dump_data($items));
+        $self->debug("[$self] $class DATA: ", $self->dump_data($data));
+    }
+
     # store hash lookup table marking valid items
     $items = $self->{ item } = {
-        map { $_ => 1 } 
-        map { split DELIMITER } 
-        @$items 
+        map { $_ => 1 }
+        keys %$data,
+        map { split DELIMITER }
+        @$items
     };
 
     # load up all the configuration items from package variables
@@ -50,13 +61,14 @@ sub init {
     # be merged in with the code in Badger::Class::Config, or rather B:C:C
     # should define a config schema.
     foreach my $item (keys %$items) {
+        next if exists $data->{ $item };
         $data->{ $item } = $config->{ $item }
             || $class->any_var( uc $item );
         $self->debug("config set $item => ", $data->{ $item }, "\n") if DEBUG;
     }
-    
+
     if (DEBUG) {
-        $self->debug("config items: ", $self->dump_data($self->{ items }));
+        $self->debug("config items: ", $self->dump_data($self->{ item }));
         $self->debug("config data: ", $self->dump_data($self->{ data }));
     }
 
@@ -65,62 +77,140 @@ sub init {
 
 
 sub get {
-    my $self  = shift;
-    my @names = map { ref $_ eq ARRAY ? @$_ : split /\W+/ } @_;
+    my $self  = shift->prototype;
+    my @names = map { ref $_ eq ARRAY ? @$_ : split /\./ } @_;
     my $name  = shift @names;
-    my $data  = $self->{ data }->{ $name };
-    my ($last, $method);
 
-    $self->debug("get: ", join(' / ', @names)) if DEBUG;
-    
-    while (defined $data && @names) {
-        $data = $data->() if ref $data eq CODE;
-        $name = shift @names;
-        if (ref $data eq HASH) {
-            $data = $data->{ $name };
+    $self->debug(
+        "get: [",
+        join('].[', $name, @names),
+        "]"
+    ) if DEBUG;
+
+    # fetch the head item
+    my $data = $self->head($name);
+
+    if (! defined $data) {
+        return $self->decline_msg(
+            no_config => $name
+        );
+    }
+
+    return @names
+        ? $self->dot($name, $data, \@names)
+        : $data;
+}
+
+sub dot {
+    my ($self, $name, $data, $dots) = @_;
+    my @done = ($name);
+    my ($dot, $last, $method);
+
+    $self->debug(
+        "dot: [",
+        join('].[', $name, @$dots),
+        "]"
+    ) if DEBUG;
+
+
+    # resolve any dotted paths after the head
+    foreach $dot (@$dots) {
+        # call any function reference to return a value
+        if (ref $data eq CODE) {
+            $data = $data->();
         }
-        elsif (ref $data eq ARRAY && numlike $name) {
-            $data = $data->[$name];
-        }
-        elsif (blessed $data && (my $method = $data->can($name))) {
-            $data = $method->($data);
-        }
-        else {
-            return $self->error_msg(
-                get => $last, $name, ref $last || 'text'
+
+        CHECK: {
+            if (ref $data eq HASH) {
+                $data = $data->{ $dot };
+                last CHECK;
+            }
+            elsif (ref $data eq ARRAY) {
+                if (numlike $dot) {
+                    $data = $data->[$dot];
+                    last CHECK;
+                }
+                # else vmethods?
+            }
+            elsif (blessed $data) {
+                if ($method = $data->can($dot)) {
+                    $data = $method->($dot);
+                    last CHECK;
+                }
+            }
+            return $self->decline_msg(
+                no_config => join('.', @done, $dot)
             );
         }
+
+        if (! defined $data) {
+            return $self->decline_msg(
+                no_config => join('.', @done, $dot)
+            );
+        }
+        push(@done, $dot);
     }
-    
+
     return $data;
 }
 
+sub head {
+    my ($self, $name) = @_;
+    # subclasses can do something more complicated
+    return $self->{ data }->{ $name };
+}
 
 sub set {
-    my $self = shift;
+    my $self = shift->prototype;
     my $name = shift;
     my $data = @_ == 1 ? shift : { @_ };
-    $self->{ data }->{ $name } = $data;
-    $self->{ item }->{ $name } = 1;
+    $self->{ data }->{ $name }   = $data;
+    $self->{ item }->{ $name } ||= 1;
     return $data;
 }
 
+sub data {
+    my $self = shift->prototype;
+    my $data = $self->{ data };
+    extend($data, @_) if @_;
+    return $data;
+}
 
 sub can_configure {
     my ($self, $name) = @_;
 
     $self = $self->prototype unless ref $self;
 
-    return 
-        unless $name && $self->{ item }->{ $name };
+    $self->debug("can_configure($name)") if DEBUG;
+
+    return
+        unless $name && $self->has_item($name);
 
     return sub {
         return @_ > 1
             ? shift->set( $name => @_ )     # set
-            : $self->{ data }->{ $name };   # get
+            : shift->get( $name );
     };
 }
 
+sub has_item {
+    my $self = shift->prototype;
+    my $name = shift;
+    my $item = $self->{ item }->{ $name };
+    if (defined $item) {
+        # A 1/0 entry in the item tells us if an item categorically does or
+        # doesn't exist in the config data set (or allowable set - it might
+        # be a valid configuration option that simply hasn't been set yet)
+        return $item;
+    }
+    else {
+        # Otherwise the existence (or not) of an item in the data set is
+        # enough to satisfy us one way or another
+        return exists $self->{ data }->{ $name };
+    }
+}
+
+
 1;
 
 __END__
@@ -132,7 +222,7 @@ Badger::Config - configuration module
 =head1 SYNOPSIS
 
     use Badger::Config;
-    
+
     my $config = Badger::Config->new(
         user => {
             name => {
@@ -149,11 +239,11 @@ Badger::Config - configuration module
             description => 'Mostly Harmless',
         },
     );
-    
+
     # fetch top-level data item - these both do the same thing
     my $user = $config->user;                       # shortcut method
     my $user = $config->get('user');                # generic get() method
-    
+
     # fetch nested data item - these all do the same thing
     print $config->get('user', 'name', 'given');    # Arthur
     print $config->get('user.name.family');         # Dent
@@ -173,7 +263,7 @@ still incomplete, but it's being worked on.
 
 =head2 new()
 
-Constructor method to create a new L<Badger::Config> object.  Configuration 
+Constructor method to create a new L<Badger::Config> object.  Configuration
 data can be specified as the C<data> named parameter:
 
     my $config = Badger::Config->new(
@@ -183,7 +273,7 @@ data can be specified as the C<data> named parameter:
         },
     );
 
-The C<items> parameter can be used to specify the names of other 
+The C<items> parameter can be used to specify the names of other
 valid configuration values that this object supports.
 
     my $config = Badger::Config->new(
@@ -248,11 +338,11 @@ example:
     # a trivial object class
     package Example;
     use base 'Badger::Base';
-    
+
     sub wibble {
         return 'wobble';
     }
-    
+
     package main;
 
     # a config with a function that returns a hash containing an object
@@ -267,7 +357,7 @@ example:
 
 =head2 set($name,$value)
 
-Method to store a value in the configuration.  
+Method to store a value in the configuration.
 
     $config->set( friend  => 'Ford Prefect' );
     $config->set( friends => ['Ford Prefect','Trillian','Marvin'] );
@@ -279,8 +369,8 @@ the L<get()> method does.
 
 =head2 can_configure($name)
 
-Internal method used to generate accessor methods on demand.  This is 
-installed using the L<auto_can|Badger::Class/auto_can> hook in 
+Internal method used to generate accessor methods on demand.  This is
+installed using the L<auto_can|Badger::Class/auto_can> hook in
 L<Badger::Class>.
 
 =head1 AUTHOR
