@@ -6,13 +6,13 @@
 #   Factory module for loading and instantiating other modules.
 #
 # NOTE
-#   This module has grown organically to fit a number of (possibly 
-#   conflicting) needs.  It needs to be completely refactored, and 
-#   probably split into a number of different factory modules.  The 
+#   This module has grown organically to fit a number of (possibly
+#   conflicting) needs.  It needs to be completely refactored, and
+#   probably split into a number of different factory modules.  The
 #   TT3 code on which this was originally based had separate base class
-#   factory modules for modules (that just got loaded), objects (that 
+#   factory modules for modules (that just got loaded), objects (that
 #   got loaded and instantiated) and single objects (that got loaded,
-#   created and cached).  With hindsight, it was a mistake to try and 
+#   created and cached).  With hindsight, it was a mistake to try and
 #   cram all that functionality into one module.  It should be separated
 #   into a base class module/API and a number of specialised subclasses.
 #
@@ -59,13 +59,13 @@ sub init_factory {
     my ($item, $items, $path, $map, $default);
 
     # 'item' and 'items' can be specified as config params or we look for
-    # $ITEM and $ITEMS variables in the current package or those of any 
+    # $ITEM and $ITEMS variables in the current package or those of any
     # base classes.  NOTE: $ITEM and $ITEMS must be in the same package
     unless ($item = $config->{ item }) {
         foreach my $pkg ($class->heritage) {
             no strict   REFS;
             no warnings ONCE;
-            
+
             if (defined ($item = ${ $pkg.PKG.ITEM })) {
                 $items = ${ $pkg.PKG.ITEMS };
                 last;
@@ -81,12 +81,13 @@ sub init_factory {
     my $ipath    = $item.PATH_SUFFIX;
     my $inames   = $item.NAMES_SUFFIX;
     my $idefault = $item.DEFAULT_SUFFIX;
-    
-    # Merge all XXXX_PATH package vars with any 'xxxx_path' or 'path' config 
+
+    # Merge all XXXX_PATH package vars with any 'xxxx_path' or 'path' config
     # items.  Ditto for XXXX_NAME / 'xxxx_name' / 'aka' and  XXXXS/ 'xxxxs'
-    
+
     my @path  = @$config{ path  => lc $ipath  };
     my @names = @$config{ names => lc $inames };
+
     $self->{ path     } = $class->list_vars(uc $ipath, @path);
     $self->{ names    } = $class->hash_vars(uc $inames, @names);
     $self->{ $items   } = $class->hash_vars(uc $items, $config->{ $items });
@@ -99,7 +100,7 @@ sub init_factory {
 
     # see if a 'xxxx_default' or 'default' configuration option is specified
     # or look for the first XXXX_DEFAULT or DEFAULT package variable.
-    $default = $config->{ $idefault } 
+    $default = $config->{ $idefault }
             || $config->{ default }
             || $class->any_var_in( uc $idefault, uc DEFAULT );
     if ($default) {
@@ -118,14 +119,14 @@ sub init_factory {
 
 sub path {
     my $self = shift->prototype;
-    return @_ 
+    return @_
         ? ($self->{ path } = ref $_[0] eq ARRAY ? shift : [ @_ ])
         :  $self->{ path };
 }
 
 sub default {
     my $self = shift->prototype;
-    return @_ 
+    return @_
         ? ($self->{ default } = $self->{ names }->{ default } = shift)
         :  $self->{ default };
 }
@@ -145,12 +146,12 @@ sub item {
     my ($type, @args) = $self->type_args(@_);
 
     # In most cases we're expecting $type to be a name (e.g. Table) which we
-    # lookup in the items hash, or tack onto one of the module bases in the 
-    # path (e.g. Template::Plugin) to create a full module name which we load 
-    # and instantiate (e.g. Template::Plugin::Table).  However, the name might 
-    # be explicitly mapped to a  reference of some kind, or the $type passed 
-    # in could already be a reference (e.g. Template::TT2::Filters allow the 
-    # first argument to be a code ref or object which implements the required 
+    # lookup in the items hash, or tack onto one of the module bases in the
+    # path (e.g. Template::Plugin) to create a full module name which we load
+    # and instantiate (e.g. Template::Plugin::Table).  However, the name might
+    # be explicitly mapped to a  reference of some kind, or the $type passed
+    # in could already be a reference (e.g. Template::TT2::Filters allow the
+    # first argument to be a code ref or object which implements the required
     # filtering behaviour).  In which case, we bypass any name-based lookup
     # and skip straight onto the "look what I found!" phase
 
@@ -158,11 +159,11 @@ sub item {
         unless textlike $type;
 
     $type = $type . '';     # auto-stringify any textlike objects
-    
-    # OK, so $type is a string.  We'll also create a canonical version of the 
+
+    # OK, so $type is a string.  We'll also create a canonical version of the
     # name (lower case dotted) to provide a case/syntax insensitve fallback
     # (e.g. so "foo.bar" can match against "Foo.Bar", "Foo::Bar" and so on)
-    
+
     my $items = $self->{ $self->{ items } };
     my $canon = dotid $type;
 
@@ -174,10 +175,10 @@ sub item {
         if exists $items->{ $type }
            && not $items->{ $type };
 
-    my $item = $items->{ $type  } 
+    my $item = $items->{ $type  }
             || $items->{ $canon }
             # TODO: this needs to be defined-or, like //
-            # Plugins can return an empty string to indicate that they 
+            # Plugins can return an empty string to indicate that they
             # do nothing.
             # HMMM.... or does it?
             ||  $self->find($type, \@args)
@@ -193,12 +194,22 @@ sub item {
 sub type_args {
     # Simple method to grok $type and @args from argument list.  The only
     # processing it does is to set $type to 'default' if it is undefined or
-    # false. Subclasses can re-define this to insert their own type mapping or 
-    # argument munging, e.g. to inject values into the configuration params 
+    # false. Subclasses can re-define this to insert their own type mapping or
+    # argument munging, e.g. to inject values into the configuration params
     # for an object
     shift;
     my $type = shift || DEFAULT;
-    return ($type, @_);
+    my @args;
+
+    if (ref $type eq HASH) {
+        @args = ($type, @_);
+        $type = $type->{ type } || DEFAULT;
+    }
+    else {
+        @args = @_;
+    }
+
+    return ($type, @args);
 }
 
 sub find {
@@ -206,11 +217,11 @@ sub find {
     my $type   = shift;
     my $bases  = $self->path;
     my $module;
-    
+
     # run the type through the type map to handle any unusual capitalisation,
     # spelling, aliases, etc.
     $type = $self->{ names }->{ $type } || $type;
-    
+
     foreach my $base (@$bases) {
         return $module
             if $module = $self->load( $self->module_names($base, $type) );
@@ -221,12 +232,12 @@ sub find {
 
 sub load {
     my $self   = shift;
-    my $loaded = $self->{ loaded }; 
+    my $loaded = $self->{ loaded };
 
     foreach my $module (@_) {
         # see if we've previously loaded a module with this name (true
         # value) or failed to load a module (defined but false value)
-            
+
         if ($loaded->{ $module }) {
             $self->debug("$module has been previously loaded") if DEBUG;
             return $module;
@@ -234,22 +245,22 @@ sub load {
         elsif (defined $loaded->{ $module }) {
             next;
         }
-                        
+
         no strict REFS;
         $self->debug("attempting to load $module") if DEBUG;
 
-        # Some filesystems are case-insensitive (like Apple's HFS), so an 
-        # attempt to load Badger::Example::foo may succeed, when the correct 
+        # Some filesystems are case-insensitive (like Apple's HFS), so an
+        # attempt to load Badger::Example::foo may succeed, when the correct
         # package name is actually Badger::Example::Foo.  We double-check
         # by looking for $VERSION or @ISA.  This is a bit dodgy because we might be
-        # loading something that has no ISA.  Need to cross-check with 
+        # loading something that has no ISA.  Need to cross-check with
         # what's going on in Badger::Class _autoload()
 
         my $loadname;
         if ( ($loadname = class($module)->maybe_load) ) {
             $self->debug("loaded $module") if DEBUG;
             $loaded->{ $module } = $loadname;
-            return $module 
+            return $module
         }
 
         $self->debug("failed to load $module") if DEBUG;
@@ -261,26 +272,26 @@ sub load {
 
 sub found {
     my ($self, $type, $item, $args) = @_;
-    
+
     if (ref $item) {
         # if it's a reference we found then forward it onto the appropriate
-        # method, e.g found_array(), found_hash(), found_code().  Fall back 
+        # method, e.g found_array(), found_hash(), found_code().  Fall back
         # on found_ref()
         my $iref = blessed($item)
-            ? OBJECT 
+            ? OBJECT
             : lc ref $item;
 
         $self->debug(
-            "Looking for handler methods: ", 
-            FOUND,'_'.$iref, "() or ", 
+            "Looking for handler methods: ",
+            FOUND,'_'.$iref, "() or ",
             FOUND_REF, "()"
         ) if DEBUG;
-        
-        my $method 
+
+        my $method
              = $self->can(FOUND . '_' . $iref)
             || $self->can(FOUND_REF)
             || return $self->error_msg( bad_ref => $self->{ item }, $type, $iref );
-            
+
         $item = $method->($self, $type, $item, $args);
     }
     else {
@@ -291,7 +302,7 @@ sub found {
     # NOTE: an item can be defined but false, e.g. a Template::Plugin which
     # return '' from its new() method to indicate it does nothing objecty
     return unless defined $item;
-    
+
     $self->debug("Found result: $type => $item") if DEBUG;
 
     # TODO: what about caching result?  Do we always leave that to subclasses?
@@ -299,9 +310,9 @@ sub found {
 }
 
 sub found_module {
-    # This method is called when a module name is found, either by being 
+    # This method is called when a module name is found, either by being
     # predefined in the factory entry table, or loaded on demand from disk.
-    # It ensures the module is loaded and and instantiates an object from the 
+    # It ensures the module is loaded and and instantiates an object from the
     # class name
     my ($self, $type, $module, $args) = @_;
     $self->debug("Found module: $type => $module") if DEBUG;
@@ -310,6 +321,9 @@ sub found_module {
 }
 
 sub found_array {
+    # This method is called when an ARRAY reference is found.  We assume that
+    # the first item is the module name (which needs to be loaded) and the
+    # second item is the class name (which needs to be instantiated).
     my ($self, $type, $item, $args) = @_;
     my ($module, $class) = @$item;
     $self->{ loaded }->{ $module } ||= class($module)->load;
@@ -332,7 +346,7 @@ sub construct {
 
 sub module_names {
     my $self = shift;
-    my @bits = 
+    my @bits =
         map { camel_case($_) }
         map { split /[\.]+/ } @_;
 
@@ -381,7 +395,7 @@ sub AUTOLOAD {
     $self = $self->prototype unless ref $self;
 
     $self->debug("factory item: $self->{ item }") if DEBUG;
-    
+
     if ($name eq $self->{ item }) {
         $self->class->method( $name => $self->can('item') );
     }
@@ -396,7 +410,7 @@ sub AUTOLOAD {
         my $class = ref $self || $self;
         die $self->message( bad_method => $name, $class, $file, $line ), "\n";
     }
-    
+
     # should be installed now
     $self->$name(@args);
 }
@@ -417,14 +431,14 @@ automatically load modules and instantiate objects on demand.
 
     package My::Widgets;
     use base 'Badger::Factory';
-    
+
     # tell the base class factory what we create
     our $ITEM        = 'widget';
     our $ITEMS       = 'widgets';
-    
+
     # define module search path for widgets
     our $WIDGET_PATH = ['My::Widget', 'Your::Widget'];
-    
+
     # lookup table for any non-standard spellings/capitalisations/paths
     our $WIDGETS     = {
         url   => 'My::Widget::URL',       # non-standard capitalisation
@@ -435,10 +449,10 @@ automatically load modules and instantiate objects on demand.
 You can then use it like this:
 
     use My::Widgets;
-    
+
     # class methods (note: widget() is singular)
     $w = My::Widgets->widget( foo => { msg => 'Hello World' } );
-    
+
     # same as:
     use My::Widget::Foo;
     $w = My::Widget::Foo({ msg => 'Hello World' });
@@ -448,7 +462,7 @@ You can then use it like this:
         extra => 'Another::Widget::Module',
         super => 'Golly::Gosh',
     );
-    
+
     # now load and instantiate new widget modules
     $w = My::Widgets->widget( extra => { msg => 'Hello Badger' } );
 
@@ -461,14 +475,14 @@ You can also create factory objects:
             super => 'Golly::Gosh',
         }
     );
-    
+
     $w = $factory->widget( foo => { msg => 'Hello World' } );
 
 The L<Badger::Factory::Class> module can be used to simplify the process
 of defining factory subclasses.
 
     package My::Widgets;
-    
+
     use Badger::Factory::Class
         item    => 'widget',
         path    => 'My::Widget Your::Widget';
@@ -481,7 +495,7 @@ of defining factory subclasses.
 
 This module implements a base class factory object for loading modules and
 instantiating objects on demand. It originated in the L<Template::Plugins>
-module, evolved over time in various directions for other projects, and was 
+module, evolved over time in various directions for other projects, and was
 eventually pulled back into line to become C<Badger::Factory>.
 
 =head2 Defining a Factory Module
@@ -493,7 +507,7 @@ define a factory for them like so:
 
     package My::Widgets;
     use base 'Badger::Factory';
-    
+
     our $ITEM           = 'widget';
     our $ITEMS          = 'widgets';
     our $WIDGET_PATH    = ['My::Widget', 'Your::Widget'];
@@ -513,20 +527,20 @@ define a factory for them like so:
 
 The C<$ITEM> and C<$ITEMS> package variables are used to define the
 singular and plural names of the items that the factory is responsible for.
-In this particular case, the C<$ITEMS> declaration isn't strictly 
-necessary because the module would correctly "guess" the plural name 
-C<widgets> from the singular C<widget> defined in C<$ITEM>.  However, 
-this is only provided as a convenience for those English words that 
+In this particular case, the C<$ITEMS> declaration isn't strictly
+necessary because the module would correctly "guess" the plural name
+C<widgets> from the singular C<widget> defined in C<$ITEM>.  However,
+this is only provided as a convenience for those English words that
 pluralise regularly and shouldn't be relied upon to work all the time.
 See the L<pluralise()|Badger::Utils/pluralise()> method in L<Badger::Utils>
 for further information, and explicitly specify the plural in C<$ITEMS> if
 you're in any doubt.
 
 The C<$WIDGET_PATH> is used to define one or more base module names under
-which your widgets are located.  The name of this variable is derived 
+which your widgets are located.  The name of this variable is derived
 from the upper case item name in C<$ITEM> with C<_PATH> appended.  In this
-example, the factory will look for the C<Foo::Bar> module as either 
-C<My::Widget::Foo::Bar> or C<Your::Widget::Foo::Bar>.  
+example, the factory will look for the C<Foo::Bar> module as either
+C<My::Widget::Foo::Bar> or C<Your::Widget::Foo::Bar>.
 
 The C<$WIDGET_DEFAULT> specifies the default item name to use if a request
 is made for a module using an undefined or false name.  If you don't specify
@@ -552,7 +566,7 @@ variable.
 Now that you've define a factory module you can use it like this.
 
     use My::Widgets;
-    
+
     my $widgets = My::Widgets->new;
     my $foo_bar = $widgets->widget('Foo::Bar');
 
@@ -568,7 +582,7 @@ Although it's not I<strictly> equivalent because the factory could just
 has easily have loaded it from C<Your::Widget::Foo::Bar> in the case that
 C<My::Widget::Foo::Bar> doesn't exist.
 
-You can specify additional arguments that will be forwarded to the object 
+You can specify additional arguments that will be forwarded to the object
 constructor method.
 
     my $foo_bar = $widgets->widget('Foo::Bar', x => 10, y => 20);
@@ -639,7 +653,7 @@ Used to fetch or update the lookup table for mapping names to modules.
     my $items = $widgets->items;
     $widgets->items( foo => 'My::Plugin::Foo' );
 
-Calling the method with arguments (named parameters or a hash reference) 
+Calling the method with arguments (named parameters or a hash reference)
 will add the new definitions into the existing table.
 
 This method can also be aliased by the plural name defined in C<$ITEMS>
@@ -702,7 +716,7 @@ see if the module can be found under that prefix.
 
 This method is called to dynamically load a module.  It iterates through
 each of the module name passed as arguments until it successfully loads
-one.  At that point it returns the module name that was successfully 
+one.  At that point it returns the module name that was successfully
 loaded and ignores the remaining arguments.  If none of the modules can
 be loaded then it returns C<undef>
 
@@ -713,7 +727,7 @@ C<items> lookup table, or by a call to L<find()>. The L<$item> argument is
 usually a module name that is forwarded onto L<found_module()>. However, it
 can also be a reference which will be forwarded onto one of the following
 methods depending on its type: L<found_array()>, L<found_hash()>,
-L<found_scalar()>, L<found_object()> (and in theory, C<found_regex()>, 
+L<found_scalar()>, L<found_object()> (and in theory, C<found_regex()>,
 C<found_glob()> and maybe others, but they're not implemented).
 
 The result returned by the appropriate C<found_XXXXX()> method will then
@@ -742,7 +756,7 @@ If the C<wizbang> widget is requested from the C<My::Widgets> factory
 in the example above, then the L<found()> method will call C<found_array()>,
 passing the array reference as an argument.
 
-The module listed in the first element is loaded.  The class name in 
+The module listed in the first element is loaded.  The class name in
 the second element is then used to instantiate an object.
 
 =head2 found_hash(\%hash)
@@ -776,7 +790,7 @@ to do something more interesting.
 
 =head2 module_names($type)
 
-This method performs the necessary mapping from a requested module name to 
+This method performs the necessary mapping from a requested module name to
 its canonical form.
 
 =head2 not_found($name,@args)
